import ij.*;

import ij.gui.GenericDialog;
import org.scijava.command.Command;
import org.scijava.log.LogService;
import org.scijava.plugin.Parameter;
import org.scijava.plugin.Plugin;


@Plugin(type = Command.class, menuPath = "Plugins > Time-Series > Merge 1 + T")
public class MergeTimeStacks1PlusT implements Command {

    @Parameter
    private LogService log = null;

    /* Static config */
    private static int defaultImg1Idx = 0;
    private static int defaultImg2Idx = 1;
    private static String[] chChoices = new String[]{"CH 1", "CH 2", "CH 3", "CH 4", "All"};

    /* Plugin parameter and default values */
    private String defaultChImg1 = "CH 1";
    private String defaultChImg2 = "CH 1";
    private int idx1 = 0;
    private int idx2 = 0;
    private ImagePlus img1 = null;
    private ImagePlus img2 = null;
    private int[] chs1 = null;
    private int[] chs2 = null;


    /**
     * Method to do all the work.
     */
    @Override
    public void run() {
        // Configure
        int status = configure();
        if (status == 1) {
            log.error("You need an open image mask an two channel images.");
            return;
        } else if (status == 2) {
            log.debug("Configuration dialog of the plugin 'Merge 1+T' was cancelled.");
            return;
        }

        // Sanity checks
        if (idx1 == idx2) {
            log.error("The stack selections can't be identical. Please select different stacks for the merging.");
            return;
        }
        if (img1.getWidth() != img2.getWidth() || img1.getHeight() != img2.getHeight()) {
            log.error("The stacks have to have the same image height and width.");
            return;
        }
        if (img1.getNSlices() != img2.getNSlices()) {
            log.error("The image files have to have the same number of slices (stack size)");
            return;
        }

        // Merge
        ImageStack stk = new ImageStack(img1.getWidth(), img2.getHeight());
        for (int f = 1; f <= img2.getNFrames(); f++) {

            for (int s = 1; s <= img2.getNSlices(); s++) {

                for (int c1 : chs1) {
                    img1.setPosition(c1, s, 0);
                    stk.addSlice(img1.getProcessor());
                    log.info("add f=" + f + ", s=" + s + ", c=" + c1 + "from input 1");
                }

                for (int c2 : chs2) {
                    img2.setPosition(c2, s, f);
                    stk.addSlice(img2.getProcessor());
                    log.info("add f=" + f + ", s=" + s + ", c=" + c2 + "from input 2");
                }
            }
        }

        ImagePlus img = new ImagePlus("merge", stk);
        int chs = chs1.length + chs2.length;
        img.setDimensions(chs, img2.getNSlices(), img2.getNFrames());

        CompositeImage comp = new CompositeImage(img, CompositeImage.COLOR);
        comp.show();
    }

    /**
     * Utility to get a list of channels to process from the input
     *
     * @param in input string
     * @return channel index list
     */
    private int[] getChannelIndices(String in) {
        int[] chs;
        if (in.equals("All")) {
            chs = new int[]{1, 2, 3, 4, 5};
        } else {
            chs = new int[1];
            chs[0] = Integer.parseInt(in.substring(3));
        }

        return chs;
    }

    /**
     * Configuration dialog
     *
     * @return status
     */
    private int configure() {
        // Check if there are open images
        final int[] idList = WindowManager.getIDList();
        if ( idList == null || idList.length < 2 ) {

            return 1;
        }

        // Get the list of open images
        final String[] imgList = new String[ idList.length ];
        for ( int i = 0; i < idList.length; ++i ) {
            imgList[ i ] = WindowManager.getImage(idList[i]).getTitle();
        }

        // Create
        final GenericDialog gd = new GenericDialog("Merge 1 + T");
        gd.addChoice("Single time-point stack", imgList, imgList[defaultImg1Idx]);
        gd.addChoice("   Channel selection", chChoices, defaultChImg1);
        gd.addChoice("Time series stack", imgList, imgList[defaultImg2Idx] );
        gd.addChoice("   Channel selection", chChoices, defaultChImg2);

        gd.showDialog();
        if (gd.wasCanceled()) {
            return 2;
        }

        // Harvest the input
        idx1 = idList[gd.getNextChoiceIndex()];
        img1 = WindowManager.getImage(idx1);
        chs1 = getChannelIndices(gd.getNextChoice());
        idx2 = idList[gd.getNextChoiceIndex()];
        img2 = WindowManager.getImage(idx2);
        chs2 = getChannelIndices(gd.getNextChoice());

        return 0;
    }
}
